@@ -176,7 +176,7 @@ GM_BrowserUI.showInstallBanner = function(browser) {
       null /* no popuup */,
       "top",
       true /* show close button */,
-      "I" /* access key */);
+      this.bundle.getString("greeting.btnAccess") /* access key */);
   } else {
     // Firefox 2.0+
     var notificationBox = this.tabBrowser.getNotificationBox(browser);
@@ -197,7 +197,7 @@ GM_BrowserUI.showInstallBanner = function(browser) {
       notificationBox.PRIORITY_WARNING_MEDIUM,
       [{
         label: this.bundle.getString("greeting.btn"),
-        accessKey: "I",
+        accessKey: this.bundle.getString("greeting.btnAccess"),
         popup: null,
         callback: GM_hitch(this, "installCurrentScript")
       }]
@@ -470,8 +470,9 @@ function GM_showPopup(aEvent) {
 
   // remove all the scripts from the list
   for (var i = popup.childNodes.length - 1; i >= 0; i--) {
-    if (popup.childNodes[i].script) {
-      popup.removeChild(popup.childNodes[i]);
+    var node = popup.childNodes[i];
+    if (node.script || node.getAttribute("value") == "hack") {
+      popup.removeChild(node);
     }
   }
 
@@ -494,9 +495,11 @@ function GM_showPopup(aEvent) {
   // build the new list of scripts
   if (runsFramed.length) {
     runsFramed.forEach(appendScriptToPopup);
-    var separator = document.createElement("menuseparator");
-    separator.setAttribute("value", "hack"); // to get removed in the loop above
-    popup.insertBefore(separator, tail);
+    if (runsOnTop.length) { // only add the separator if there is stuff below
+      var separator = document.createElement("menuseparator");
+      separator.setAttribute("value", "hack"); // remove it in the loop above
+      popup.insertBefore(separator, tail);
+    }
   }
   runsOnTop.forEach(appendScriptToPopup);
 
